# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import template
from django.template.loader import get_template
from django.forms.widgets import flatatt

try:
    from django.utils.encoding import force_text
except ImportError:
    from django.utils.encoding import force_unicode as force_text

from ..settings import (
    datatable_css_url, filechooser_css_url, jquery_js_url, datatable_js_url, filechooser_js_url, get_filechooser_setting
)

register = template.Library()

@register.simple_tag(takes_context=True)
def filechooser_filetable(context, id):
    """
    Return HTML for the datatabel which is used to display the files
    **Tag name**::
        filechooser_filetable
    **usage**::
        {% filechooser_filetable id="" %}
    **example**::
        {% filechooser_filetable id="filelist" %}
    """

    context['filechooser_id'] = id

    return get_template('filechooser/filetable.html').render(context)

@register.simple_tag
def filechooser_css():
    """
    Return HTML for the (inclusion of) the filechooser CSS.
    **Tag name**::
        filechooser_css
    **usage**::
        {% filechooser_css %}
    **example**::
        {% filechooser_css %}
    """
    urls = [url for url in [datatable_css_url(), filechooser_css_url()] if url]
    return ''.join([render_css_tag(url) for url in urls])


@register.simple_tag(takes_context=True)
def filechooser_javascript(context, id, jquery = None, namespace = None):
    """
    Return HTML for filechooser JavaScript, which also means the js
	for the datatable and optionally the jquery.
    Adjust url in settings. If no url is returned, we don't want this
    statement to return any HTML.
    This is intended behavior.
    Default value: ``None``
    This value is configurable, see Settings section
    **Tag name**::
        filechooser_javascript
    **Parameters**:
        :jquery: True to include jQuery also
    **usage**::
        {% filechooser_javascript id=""%}
    **example**::
        {% filechooser_javascript id="filelist" jquery=1 %}
    """
    # See if we have to include jQuery
    if jquery is None:
        jquery = get_filechooser_setting('include_jquery', False)

    urls = [url for url in [datatable_js_url()] if url]

    if jquery:  urls.prepend(bootstrap_jquery_url())

    url = 'filechooser_' + id
    if namespace: url = namespace + ':' + url

    context['scripts'] = urls
    context['filechooser_id'] = id
    context['filechooser_url_id'] = url
    context['js_url'] = filechooser_js_url()

    return get_template('filechooser/filechooser.js').render(context)


def render_css_tag(url):
    """
    Render the css tag
    """
    return render_tag('link', attrs={'href': url, 'rel': 'stylesheet', 'media': 'all'}, self_close = True)


def render_tag(tag, attrs=None, content=None, self_close=False):
    """
    Render a given tag, based on tag-name, attrs, content and whether it should be closed or not.
    """
    builder = "<{tag}{attrs}{self_close}>{content}"
    if content or not self_close: builder += "</{tag}>"

    return builder.format(
        tag = tag,
        attrs=flatatt(attrs) if attrs else '',
        self_close='/' if self_close else '',
        content=force_text(content) if content else ''
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage as static


# Default settings
FILECHOOSER_DEFAULTS = {
    'jquery_js_url': static.url('js/jquery.min.js'),
    'datatable_js_url': static.url('js/jquery.dataTables.min.js'),
    'filechooser_js_url': static.url('filechooser/js/filechooser.js'),
    'datatable_css_url': static.url('css/jquery.dataTables.min.css'),
    'filechooser_css_url': static.url('filechooser/css/filechooser.css'),
}


# Start with a copy of default settings
FILECHOOSER = FILECHOOSER_DEFAULTS.copy()

# Override with user settings from settings.py
FILECHOOSER.update(getattr(settings, 'FILECHOOSER', {}))


def get_filechooser_setting(setting, default=None):
    """
    Read a setting
    """
    return FILECHOOSER.get(setting, default)


def jquery_js_url():
    """
    Return the url to jQuery file to use
    """
    return get_filechooser_setting('jquery_js_url')


def datatable_js_url():
    """
    Return the url to dataTable jQuery file to use
    """
    return get_filechooser_setting('datatable_js_url')


def filechooser_js_url():
    """
    Return the url to filechooser js file to use
    """
    return get_filechooser_setting('filechooser_js_url')

def datatable_css_url():
    """
    Return the url to dataTable css file to use
    """
    return get_filechooser_setting('datatable_css_url')


def filechooser_css_url():
    """
    Return the url to filechooser css file to use
    """
    return get_filechooser_setting('filechooser_css_url')



import django.conf.global_settings as DEFAULT_SETTINGS

SECRET_KEY = 'test-secret-key'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}

INSTALLED_APPS = (
    'filechooser',
)

FILECHOOSER = {
    'jquery_js_url': 'some-cdn-url',
    'datatable_css_url': 'some-other-cdn-url'
}

STATIC_URL = '/static/'

MIDDLEWARE_CLASSES = DEFAULT_SETTINGS.MIDDLEWARE_CLASSES



import json

from django.test import TestCase


from filechooser.settings import (
    datatable_css_url, filechooser_css_url, jquery_js_url, datatable_js_url, filechooser_js_url
)

from django.http import HttpResponse, HttpResponseNotFound

class SettingsTestCase(TestCase):

    def test_overwritten_value(self):
        self.assertEquals(jquery_js_url(), 'some-cdn-url')
        self.assertEquals(datatable_css_url(), 'some-other-cdn-url')

    def test_non_overwritten_value(self):
        self.assertEquals(filechooser_css_url(), '/static/filechooser/css/filechooser.css')
        self.assertEquals(datatable_js_url(), '/static/js/jquery.dataTables.min.js')
        self.assertEquals(filechooser_js_url(), '/static/filechooser/js/filechooser.js')



import json

from django.test import TestCase
from filechooser.filechooser import FileChooser

from django.http import HttpResponse, HttpResponseNotFound

class FileChooserTestCase(TestCase):
    ID = 'tEsTIdEnTiFiEr'
    PATH = '.'

    def setUp(self):
        self.filechooser = FileChooser(self.ID, self.PATH, callback_for_test)


    def test_filechooser_url_pattern_name(self):
        """Validate whether the url pattern has the correct name"""
        pattern = self.filechooser.url_pattern()

        #self.assertEqual(pattern.callback, FileChooser.process)
        self.assertEqual(pattern.name, 'filechooser_' + self.ID)


    def test_filechooser_url_pattern_resolve(self):
        """Validate whether the url pattern matches the url and parameters correctly"""
        pattern = self.filechooser.url_pattern()

        ajax_list_path = pattern.resolve('filechooser/tEsTIdEnTiFiEr/ajax/list/')
        self.assertTrue(ajax_list_path)
        self.assertEqual(ajax_list_path.kwargs['type'], 'ajax')
        self.assertEqual(ajax_list_path.kwargs['method'], 'list')
        self.assertEqual(ajax_list_path.kwargs['file'], '')

        http_process_path = pattern.resolve('filechooser/tEsTIdEnTiFiEr/http/process/test.txt')
        self.assertTrue(http_process_path)
        self.assertEqual(http_process_path.kwargs['type'], 'http')
        self.assertEqual(http_process_path.kwargs['method'], 'process')
        self.assertEqual(http_process_path.kwargs['file'], 'test.txt')


    def test_filechooser_url_pattern_resolve_false(self):
        """Validate whether the url pattern does match urls which it should not"""
        pattern = self.filechooser.url_pattern()

        false_paths = {
            'filechooser/some_other_identifier/ajax/list/', # wrong identifier
            'filechooser/some_other_identifier/http/process/', # wrong identifier
            'filechooser/tEsTIdEnTiFiEr/https/list/', # wrong type
            'filechooser/tEsTIdEnTiFiEr/https/process/', # wrong type
            'filechooser/tEsTIdEnTiFiEr/ajax/non_existing/', # wrong method
            'filechooser/tEsTIdEnTiFiEr/http/non_existing/', # wrong method
        }

        for path in false_paths:
            self.assertFalse(pattern.resolve(path))


    def test_filechooser_process_list(self):
        """Validate whether the processing of a list works"""
        result_json = self.filechooser.process(None, '', 'ajax', 'list').content.decode('utf8')

        result = json.loads(result_json)
        self.assertTrue(result_json)
        self.assertTrue(result['data'])
        self.assertEqual(result['data'][0]['order'], 'a.git')
        self.assertEqual(result['data'][0]['filename']['display'], '.git')
        self.assertEqual(result['data'][0]['filename']['type'], 'folder')
        self.assertEqual(result['data'][0]['size']['display'], '-')
        self.assertTrue(result['data'][0]['mtime'])


    def test_filechooser_process_process(self):
        """Validate whether the processing of a process request works"""
        result = self.filechooser.process(None, '','http', 'process')

        self.assertEqual(result, 'some_unique_info_from_callback')


    def test_filechooser_process_false(self):
        """Validate whether the processing of incorrect requests is correctly handled"""
        result_wrong_combination = self.filechooser.process(None, '', 'http', 'list')

        self.assertIsInstance(result_wrong_combination, HttpResponseNotFound)

        result = None
        with self.assertRaises(EnvironmentError) as cm:
             result = self.filechooser.process(None, '../../', 'ajax', 'list')

        exception = cm.exception
        self.assertFalse(result)
        self.assertEquals(str(exception), 'Do not try to escape from the designated folder')



def callback_for_test(filename):
    return 'some_unique_info_from_callback'



import datetime
import json
import humanize
import os
import urllib

from operator import itemgetter

from django.http import HttpResponse, HttpResponseNotFound
from django.conf.urls import url

class FileChooser(object):
    def __init__(self, id, basedir, callback = None):
        self.pattern = r'^filechooser/'+id+'/(?P<type>((ajax)|(http)))/(?P<method>((list)|(process)))/(?P<file>.*)$'
        self.id = id

        # check that id only contains a-z0-9

        self.callback = callback
        self.basedir = os.path.abspath(basedir)

    def url_pattern(self):
        return url(self.pattern, self.process, name='filechooser_'+self.id)

    def process(self, request, file=None, type=None, method=None):
        # make sure we can't get higher than the given basedir
        path = os.path.join(self.basedir, file)
        absolute = os.path.abspath(path)

        # path should still contain at least the BASE directory
        if not absolute.startswith(self.basedir):
            raise  EnvironmentError("Do not try to escape from the designated folder")

        if method == 'list' and type == 'ajax':
            return self.__ajax_list(file)
        elif method == 'process' and type == 'http':
            if self.callback: return self.callback(file)
        else:
            return HttpResponseNotFound("<h1>Requested operation not supported in FileChooser</h1>")


    def __ajax_list(self, folder):
        path = os.path.join(self.basedir, folder)
        records = []

        for filename in os.listdir(path):
            records.append(self.__process_file(folder, filename))

        result = {
            "data": sorted(records, key=itemgetter('order'))
        }

        return HttpResponse(json.dumps(result))

    def __process_file(self, folder, filename):
        fullpath = os.path.join(self.basedir, folder, filename)

        mtime = os.path.getmtime(fullpath)
        mtime_date = datetime.datetime.fromtimestamp(mtime)

        size = os.path.getsize(fullpath)
        natural_size = humanize.naturalsize(size,gnu=True)
        sortkey = 'z'
        icon = filetype = 'file'

        if os.path.isdir(fullpath):
            sortkey = 'a' # force the folder in front of the files
            filetype = 'folder'
            icon = 'folder-open'
            natural_size = '-'
            size = -1

        return {
            'order'   : sortkey + os.path.join(folder, filename).lower(),
            'filename': {
                'icon':    icon,
                'value':   os.path.join(folder, filename),
                'display': filename,
                'type':    filetype,
            },
            'size': {
                'value':   size,
                'display': natural_size
            },
            'mtime': {
                'value':   mtime,
                'display': humanize.naturaltime(datetime.datetime.now() - mtime_date),
                'title':   mtime_date.strftime("%x %X")
            }
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def get_metadata():
    import re
    with open(os.path.join("filechooser", "__init__.py")) as f:
        return dict(re.findall("__([a-z]+)__ = ['\"]([^'\"]+)['\"]", f.read()))

metadata = get_metadata()

if sys.argv[-1] == 'publish':
    os.system('cd docs && make html')
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (metadata['version'], metadata['version']))
    print("  git push --tags")
    sys.exit()

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

setup(
    name='django-filechooser',
    version=metadata['version'],
    description="""jQuery Filechooser for Django projects""",
    long_description=readme + '\n\n' + history,
    author='Martin van Wingerden',
    author_email='martinvw@gmail.com',
    url='https://github.com/martinvw/django-filechooser',
    download_url="https://github.com/martinvw/django-filechooser/archive/v"+metadata['version']+".tar.gz",
    packages=[
        'filechooser',
    ],
    include_package_data=True,
    install_requires=['humanize>=0.5.0'],
    license="MIT",
    zip_safe=False,
    keywords='django-filechooser',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
)
